from __future__ import absolute_import, division, print_function
import pickle, os
import numpy as np
import sys
parent_path = os.path.dirname(os.path.realpath(__file__))
grandparent_path = parent_path

import matplotlib.pyplot as plt
import matplotlib

import seaborn as sns
sns.set_theme(style = 'darkgrid')

# import seaborn as sns
# sns.set_theme(style = 'darkgrid')

matplotlib.rcParams.update({'font.size': 12})
matplotlib.rcParams.update({'lines.linewidth': 2})
matplotlib.rcParams['pdf.fonttype'] = 42
matplotlib.rcParams['ps.fonttype'] = 42

results_dir_names = ['ShuffledPixels100_SamplesM_catoni_bcs_16', 'ShuffledPixels200_SamplesM_catoni_bcs_16', 'ShuffledPixels300_SamplesM_catoni_bcs_16']

root_saved_dir = grandparent_path
# paths_to_result_files = [root_saved_dir + name + '/runs_analysis.pkl' for name in results_dir_names]
paths_to_result_files = [root_saved_dir + '/' + name for name in results_dir_names]

legend_names = ['Permuted Pixels - 100 pixel swaps', 'Permuted Pixels - 200 pixel swaps', 'Permuted Pixels - 300 pixel swaps']

n_expirements = len(paths_to_result_files)

plt.figure()

basic_list = range(5,61, 5)
# basic_list = [1,2]
m_train_samples_vec = [ele * 1000 for ele in basic_list] # max value 60000

len_m_train_samples_vec = len(m_train_samples_vec)

for i_exp in range(n_expirements):
	i_exp_test_err_vec = np.zeros(len_m_train_samples_vec)
	i_exp_test_err_std_vec = np.zeros(len_m_train_samples_vec)
	i_exp_test_bound_vec = np.zeros(len_m_train_samples_vec)
	i_exp_test_bound_std_vec = np.zeros(len_m_train_samples_vec)
	for i_task in range(len_m_train_samples_vec):
		data_file_name = paths_to_result_files[i_exp]+'/' + str(m_train_samples_vec[i_task]) + '/' + 'run_data.pkl'
		with open(data_file_name,'rb') as f:
			data = pickle.load(f)[1]
			test_err_vec = data['test_err_vec']
			test_bound_vec = data['test_bound_vec']
			i_exp_test_err_vec[i_task] = test_err_vec.mean()
			i_exp_test_err_std_vec[i_task] = test_err_vec.std()
			i_exp_test_bound_vec[i_task] = test_bound_vec.mean()
			i_exp_test_bound_std_vec[i_task] = test_bound_vec.std()           

	n_tasks_vec = list(range(1, len_m_train_samples_vec+1))
	mean_bound_per_tasks_n = i_exp_test_bound_vec
	print(mean_bound_per_tasks_n)	
	std_bound_per_tasks_n = i_exp_test_bound_std_vec
	plt.errorbar(n_tasks_vec, mean_bound_per_tasks_n, yerr=std_bound_per_tasks_n,
                 label=legend_names[i_exp])
	plt.xticks(n_tasks_vec, list(basic_list))
	# plt.text(10.27, -0.06, '$x10^3$', ha='left') # correspond to plt.ylim(0,1)
	# plt.text(10.27, -0.15, '$x10^3$', ha='left') # correspond to plt.ylim(0,2.5)
	plt.text(12.27, -0.048, '$x10^3$', ha='left') # correspond to plt.ylim(0,1.7)



plt.legend()
plt.xlabel('Sample size per training-task', fontsize=18)
plt.ylabel('Bound on new task', fontsize=18)

# plt.savefig(root_saved_dir + '/' + 'SamplesM_plot_bound.pdf', format='pdf', bbox_inches='tight')

# plt.ylim(0,2.5) # zoom kl_bound
plt.ylim(0,0.7) # zoom catoni_bound
plt.xlim(0.5,12.9)
plt.savefig(root_saved_dir + '/' + 'SamplesM_plot_bound.pdf', format='pdf', bbox_inches='tight')

plt.show()
