from __future__ import absolute_import, division, print_function
import pickle, os
import numpy as np
import sys
parent_path = os.path.dirname(os.path.realpath(__file__))
grandparent_path = parent_path

import matplotlib.pyplot as plt
import matplotlib

import seaborn as sns
sns.set_theme(style = 'darkgrid')

matplotlib.rcParams.update({'font.size': 12})
matplotlib.rcParams.update({'lines.linewidth': 2})
matplotlib.rcParams['pdf.fonttype'] = 42
matplotlib.rcParams['ps.fonttype'] = 42

results_dir_names = ['ShuffledPixels100_TasksN_catoni_150_epoch', 'ShuffledPixels200_TasksN_catoni_150_epoch', 'ShuffledPixels300_TasksN_catoni_150_epoch']

root_saved_dir = grandparent_path + '/'
# paths_to_result_files = [root_saved_dir + name + '/runs_analysis.pkl' for name in results_dir_names]
paths_to_result_files = [root_saved_dir + name for name in results_dir_names]

legend_names = ['Permuted Pixels - 100 pixel swaps', 'Permuted Pixels - 200 pixel swaps', 'Permuted Pixels - 300 pixel swaps']

n_expirements = len(paths_to_result_files)

plt.figure()

num_training_tasks = 10

for i_exp in range(n_expirements):
	i_exp_test_err_vec = np.zeros(num_training_tasks)
	i_exp_test_err_std_vec = np.zeros(num_training_tasks)
	i_exp_test_bound_vec = np.zeros(num_training_tasks)
	i_exp_test_bound_std_vec = np.zeros(num_training_tasks)
	for i_task in range(num_training_tasks):
		data_file_name = paths_to_result_files[i_exp]+'/' + str(i_task+1) + '/' + 'run_data.pkl'
		with open(data_file_name,'rb') as f:
			data = pickle.load(f)[1]
			test_err_vec = data['test_err_vec']
			test_bound_vec = data['test_bound_vec']
			i_exp_test_err_vec[i_task] = test_err_vec.mean()
			i_exp_test_err_std_vec[i_task] = test_err_vec.std()
			i_exp_test_bound_vec[i_task] = test_bound_vec.mean()
			i_exp_test_bound_std_vec[i_task] = test_bound_vec.std()           

	n_tasks_vec = list(range(1, num_training_tasks+1))
	mean_error_per_tasks_n = i_exp_test_err_vec
	std_error_per_tasks_n = i_exp_test_err_std_vec
	plt.errorbar(n_tasks_vec, 100 * mean_error_per_tasks_n, yerr=100 * std_error_per_tasks_n,
                 label=legend_names[i_exp])
	plt.xticks(n_tasks_vec)


plt.legend()
plt.xlabel('Number of training-tasks', fontsize=18)
plt.ylabel('Error on new task [%]', fontsize=18)

# plt.savefig(root_saved_dir + 'TasksN_plot.pdf', format='pdf', bbox_inches='tight')

plt.ylim(1,8) # zoom
plt.xlim(0.5,10.5)
plt.savefig(root_saved_dir + 'TasksN_plot_error.pdf', format='pdf', bbox_inches='tight')

plt.show()
