from data_generator import DataGenerator
from tensorflow.python.platform import flags

FLAGS = flags.FLAGS

## Dataset/method options
flags.DEFINE_string('datasource', 'sinusoid', 'sinusoid or omniglot or miniimagenet')
flags.DEFINE_integer('num_classes', 5, 'number of classes used in classification (e.g. 5-way classification).')
# oracle means task id is input (only suitable for sinusoid)
flags.DEFINE_string('baseline', None, 'oracle, or None')

## Training options
flags.DEFINE_integer('pretrain_iterations', 0, 'number of pre-training iterations.')
flags.DEFINE_integer('metatrain_iterations', 15000, 'number of metatraining iterations.') # 15k for omniglot, 50k for sinusoid
flags.DEFINE_integer('meta_batch_size', 25, 'number of tasks sampled per meta-update')
flags.DEFINE_float('meta_lr', 0.001, 'the base learning rate of the generator')
flags.DEFINE_integer('update_batch_size', 5, 'number of examples used for inner gradient update (K for K-shot learning).')
flags.DEFINE_float('update_lr', 1e-3, 'step size alpha for inner gradient update.') # 0.1 for omniglot
flags.DEFINE_integer('num_updates', 1, 'number of inner gradient updates during training.')

if FLAGS.datasource == 'sinusoid':
    data_generator = DataGenerator(FLAGS.update_batch_size * 2, FLAGS.meta_batch_size)
else:
    if FLAGS.metatrain_iterations == 0 and FLAGS.datasource == 'miniimagenet':
        assert FLAGS.meta_batch_size == 1
        assert FLAGS.update_batch_size == 1
        data_generator = DataGenerator(1, FLAGS.meta_batch_size)  # only use one datapoint,
    else:
        if FLAGS.datasource == 'miniimagenet':  # TODO - use 15 val examples for imagenet?
            if FLAGS.train:
                data_generator = DataGenerator(FLAGS.update_batch_size + 15,
                                               FLAGS.meta_batch_size)  # only use one datapoint for testing to save memory
            else:
                data_generator = DataGenerator(FLAGS.update_batch_size * 2,
                                               FLAGS.meta_batch_size)  # only use one datapoint for testing to save memory
        else:
            data_generator = DataGenerator(FLAGS.update_batch_size * 2,
                                           FLAGS.meta_batch_size)  # only use one datapoint for testing to save memory