import tensorflow as tf
import numpy as np

from pacoh.algorithms import BayesianNeuralNetworkSVGD
import pacoh.datasets


N_CLASSES = 10  # K-way
DATASET = 'fmnist'

likelihood_config = {'class': {'learn_likelihood_variables': False},
                     'reg': {'log_var': 2 * np.log(0.01),
                             'learn_likelihood_variables': False},
                     'type': 'class'
                     }

model_config = {'fc': {'input_dim': None,
                       'output_dim': 1 if N_CLASSES == 2 else N_CLASSES,  # equal to the number of classes
                       'layer_units': [120, 120, 120, 1 if N_CLASSES == 2 else N_CLASSES],  # equal to output_dim
                       'activations': ['relu', 'relu', 'relu', 'sigmoid' if N_CLASSES == 2 else 'softmax']
                       },
                 'cnn': {'activations': ['elu', 'sigmoid' if N_CLASSES == 2 else 'softmax'],
                        'output_dim': 1 if N_CLASSES == 2 else N_CLASSES},
                'type': 'cnn'}

data_config = {'dataset': DATASET,
                'n_classes': N_CLASSES,
                'n_tasks_train': 1,
                'n_tasks_valid': 1,
                'n_samples_train': 5,
                'n_samples_context': 6999,
                'n_samples_test': 3000,
                'tiled': True,
                'batch_size': 10,
                'posterior_inference_batch_size': 10,
                'meta_batch_size': -1
               }


model_gaussian_prior = {'mean_mu': 0.0,
                        'bias_log_var_mu': 2 * np.log(1.0),
                        'weight_log_var_mu': 2 * np.log(0.5)
                        }

prior_config = {'gaussian': {'model': model_gaussian_prior,
                             'likelihood': {},
                             'init_std': 0.1,
                             'n_batched_priors': 1,
                             'add_noise': False
                             },
                'type': 'gaussian'
                }

posterior_config = {'particle': {'n_samples': 3,
                                 'use_init_std': True
                                 },
                    'type': 'particle'
                    }

training_config = {'epochs': 50,
                   'n_train_loss_logs': 50,
                   'n_eval_runs': 50,
                   'optimizer': 'adamw',
                   'learning_rate': 0.01,
                   'plot_functions': False,
                   'plot_pred_lines': False,
                   'plot_pred_std': False,
                   'weight_decay': 0.0,
                   'prior_prob_weight': 0.2,
                   'bandwidth': 10
                   }

prior_config = prior_config[prior_config['type']]
posterior_config = posterior_config[posterior_config['type']]

tf.config.experimental_run_functions_eagerly(True)

# create datasets and corresponding samplers
n_batched_models = posterior_config['n_samples']

_, meta_test_sampler = pacoh.datasets.get_classification_datasets(data_config, n_batched_models_test=n_batched_models,
                                                                  n_batched_models_train=n_batched_models)

dataset_sampler = meta_test_sampler.tasks[0]

class_model = BayesianNeuralNetworkSVGD(dataset_sampler, model_config, likelihood_config,
                                      prior_config, posterior_config, training_config)

class_model.fit()