import tensorflow as tf
import numpy as np

from pacoh.algorithms import BayesianNeuralNetworkSVGD
import pacoh.datasets


N_CLASSES = 5  # K-way
DATASET = 'omniglot_5shot_small'

likelihood_config = {'class': {'learn_likelihood_variables': False},
                     'reg': {'log_var': 2 * np.log(0.01),
                             'learn_likelihood_variables': False},
                     'type': 'class'
                     }

model_config = {'fc': {'input_dim': None,
                       'output_dim': 1 if N_CLASSES == 2 else N_CLASSES,  # equal to the number of classes
                       'layer_units': [120, 120, 120, 1 if N_CLASSES == 2 else N_CLASSES],  # equal to output_dim
                       'activations': ['relu', 'relu', 'relu', 'sigmoid' if N_CLASSES == 2 else 'softmax']
                       },
                 'cnn': {'activations': ['relu', 'sigmoid' if N_CLASSES == 2 else 'softmax'],
                        'output_dim': 1 if N_CLASSES == 2 else N_CLASSES},
                'type': 'cnn'}

data_config = {'dataset': DATASET,
                'n_classes': N_CLASSES,
                'n_tasks_train': None,
                'n_tasks_valid': None,
                'n_samples_train': None,
                'n_samples_context': None,
                'n_samples_test': None,
                'tiled': True,
                'batch_size': 10,
                'posterior_inference_batch_size': 10,
                'meta_batch_size': -1
               }


model_gaussian_prior = {'mean_mu': 0.0,
                        'bias_log_var_mu': 2 * np.log(1.0),
                        'weight_log_var_mu': 2 * np.log(0.5)
                        }

prior_config = {'gaussian': {'model': model_gaussian_prior,
                             'likelihood': {},
                             'init_std': 0.1,
                             'n_batched_priors': 1,
                             'add_noise': False
                             },
                'type': 'gaussian'
                }

posterior_config = {'n_samples': 5,
                    'use_init_std': True,
                    'type': 'particle'
                    }

training_config = {'n_total_iterations': 20000,
                   'n_train_loss_logs': 10,
                   'n_eval_runs': 10,
                   'optimizer': 'adamw',
                   'learning_rate': 0.001,
                   'plot_functions': False,
                   'plot_pred_lines': False,
                   'plot_pred_std': False,
                   'weight_decay': 0.0,
                   'prior_prob_weight': 1.0,
                   'bandwidth': 10
                   }

prior_config = prior_config[prior_config['type']]

# tf.config.experimental_run_functions_eagerly(True)

# create datasets and corresponding samplers
n_batched_models = posterior_config['n_samples']

_, meta_test_sampler = pacoh.datasets.get_classification_datasets(data_config, n_batched_models_test=n_batched_models,
                                                                  n_batched_models_train=n_batched_models, )


eval_metrics = []
for task_id, dataset_sampler in enumerate(meta_test_sampler.tasks):
    print('------------- TASK %i ------------'%task_id)
    dataset_sampler = dataset_sampler

    class_model = BayesianNeuralNetworkSVGD(dataset_sampler, model_config, likelihood_config,
                                          prior_config, posterior_config, training_config)

    class_model.fit()
    eval_metrics.append(class_model.eval()[0])

    import pprint
    pprint.pprint(eval_metrics)

acc = []
ll = []
cal_err = []
for metrics_per_task in eval_metrics:
    acc.append(metrics_per_task['avg_acc'])
    ll.append(metrics_per_task['avg_ll'].numpy())
    cal_err.append(metrics_per_task['cal_err'].numpy())


print('\n ------ Results averaged over tasks ------ ')
print('avg_acc', np.mean(acc), '+-', np.std(acc))
print('avg_ll', np.mean(ll), '+-', np.std(ll))
print('cal_err', np.mean(cal_err), '+-', np.std(cal_err))