import os
import sys
import hashlib

BASE_DIR = os.path.dirname(os.path.dirname(os.path.dirname(os.path.abspath(__file__))))
sys.path.append(BASE_DIR)

from pacoh.experiments.utils import AsyncExecutor, generate_launch_commands
import pacoh.experiments.bandits.bnn_bandit_exp

cluster = False

N_THREADS = 2

N_CPUS = 8

exp_config = {
    'mode': ['thomson_sampling', 'ucb'],
    'task': ['mhc_0_1', 'mhc_0_0'],
    'bandit_seed': [60, 61, 62, 63, 64],
    'n_iter': [50],
    'learn_likelihood': [True, False],
    'prior_std': [0.5, 1.0],
    'n_particles': [20],
    'n_train_iter': [5000, 10000],
    'bandwidth': [100., 1000.],
    'prior_weight': [1.0]
}

command_list = generate_launch_commands(pacoh.experiments.bandits.bnn_bandit_exp, exp_config,
                                        check_flags=False)

if cluster :
    cluster_cmds = []
    for python_cmd in command_list:
        cmd_hash = hashlib.md5(str.encode(python_cmd)).hexdigest()

        bsub_cmd = 'bsub' \
                   ' -W 3:59'\
                   ' -R "rusage[mem=1048]"' \
                   ' -n %i '% (N_THREADS)
        cluster_cmds.append(bsub_cmd + ' ' + python_cmd)
    answer = input("About to submit %i compute jobs to the cluster. Proceed? [yes/no]\n"%len(cluster_cmds))
    if answer == 'yes':
        for cmd in cluster_cmds:
            os.system(cmd)
else:
    answer = input("About to start %i compute jobs on %i workers. Proceed? [yes/no]\n" % (len(command_list), N_CPUS))
    if answer == 'yes':
        exec_fn = lambda cmd: os.system(cmd)
        executor = AsyncExecutor(n_jobs=N_CPUS)
        executor.run(exec_fn, command_list)