import os
import sys
import hashlib

BASE_DIR = os.path.dirname(os.path.dirname(os.path.dirname(os.path.abspath(__file__))))
sys.path.append(BASE_DIR)

from pacoh.experiments.utils import AsyncExecutor, generate_launch_commands
import pacoh.experiments.bandits.meta_bnn_bandit_exp

cluster = False

N_THREADS = 2

N_CPUS = 32

# mhc_0_0 corresponds to allele A_6901 --> 833 candidates / arms

exp_config = {
    'mode': ['ucb', 'thomson_sampling'],
    'task': ['mhc_0_0'],
    'bandit_seed': [60, 61, 62, 63, 64],
    'n_iter': [50],
    'learn_likelihood': [True],
    'n_particles': [20],
    'n_meta_train_iter': [5000, 20000],
    'n_train_iter': [5000, ],
    'bandwidth': [10., 100.],
    'prior_weight': [1.0],
    'meta_prior_weight': [0.1, 1.0],
    'log_var_mean': [-0.7, -0.5, -0.3, 0.0, 0.3, 0.5],
    'logsumexp': [True]
}

command_list = generate_launch_commands(pacoh.experiments.bandits.meta_bnn_bandit_exp, exp_config,
                                        check_flags=False)

if cluster :
    cluster_cmds = []
    for python_cmd in command_list:
        cmd_hash = hashlib.md5(str.encode(python_cmd)).hexdigest()

        bsub_cmd = 'bsub' \
                   ' -W 3:59'\
                   ' -R "rusage[mem=1048]"' \
                   ' -n %i '% (N_THREADS)
        cluster_cmds.append(bsub_cmd + ' ' + python_cmd)
    answer = input("About to submit %i compute jobs to the cluster. Proceed? [yes/no]\n"%len(cluster_cmds))
    if answer == 'yes':
        for cmd in cluster_cmds:
            os.system(cmd)
else:
    answer = input("About to start %i compute jobs on %i workers. Proceed? [yes/no]\n" % (len(command_list), N_CPUS))
    if answer == 'yes':
        exec_fn = lambda cmd: os.system(cmd)
        executor = AsyncExecutor(n_jobs=N_CPUS)
        executor.run(exec_fn, command_list)