import os
import pandas as pd
import numpy as np
import argparse

from config import DATA_DIR
HPARAM_EXP_DIR = '/home/jonasrothfuss/Dropbox/Publication_Data/PACOH_ICML/pacoh_nn'

def main(args):
    DATASET = args.dataset  # put dataset name here to filer results

    csv_files = [path for path in os.listdir(HPARAM_EXP_DIR) if '.csv' in path and args.prefix in path]

    for csv_name in csv_files:
        if DATASET in csv_name:

            CSV_PATH = os.path.join(HPARAM_EXP_DIR, csv_name)

            # 1) load df
            df = pd.read_csv(CSV_PATH)

            # 2) aggregate over seeds

            if 'meta_gp' in csv_name:
                hyperparams = list(
                    set(df.columns) - {'Unnamed: 0', 'random_seed', 'train_obj', 'll',
                                       'rmse', 'calib_err', 'loss', 'avg_acc'})
            elif 'MR-MAML' in csv_name:
                hyperparams = list(
                    set(df.columns) - {'Unnamed: 0', 'random_seed', 'rmse_1_steps', 'rmse_5_steps'})
            elif 'mll' in csv_name:
                hyperparams = list(
                    set(df.columns) - {'Unnamed: 0', 'random_seed', 'll', 'rmse', 'calib_err'})
            else:
                hyperparams = list(
                    set(df.columns) - {'Unnamed: 0', 'timesteps_total', 'random_seed', 'train_obj', 'avg_ll',
                                       'avg_rmse', 'cal_err', 'loss', 'avg_acc'})

            if 'omniglot' in csv_name:
                if 'maml' not in csv_name or 'bmaml' in csv_name:
                    df_aggregated = df.groupby(hyperparams).aggregate(
                        {'avg_ll': [np.mean, np.std],
                         'avg_acc': [np.mean, np.std],
                         'cal_err': [np.mean, np.std]}).round(3)
                else:
                    df_aggregated = df.groupby(hyperparams).aggregate(
                        {'avg_acc': [np.mean, np.std]})
            else:
                if 'meta_gp' in csv_name or 'meta_mll' in csv_name:
                    df_aggregated = df.groupby(hyperparams).aggregate(
                        {'ll': [np.mean, np.std],
                         'rmse': [np.mean, np.std],
                         'calib_err': [np.mean, np.std]}).round(3)
                elif 'MR-MAML' in csv_name:
                    df_aggregated = df.groupby(hyperparams).aggregate(
                        {'rmse_1_steps': [np.mean, np.std],
                         'rmse_5_steps': [np.mean, np.std]}).round(3)
                elif 'maml' not in csv_name or 'bmaml' in csv_name:
                    df_aggregated = df.groupby(hyperparams).aggregate(
                        {'avg_ll': [np.mean, np.std],
                         'avg_rmse': [np.mean, np.std],
                         'cal_err': [np.mean, np.std]}).round(3)
                else:
                    df_aggregated = df.groupby(hyperparams).aggregate(
                        {'avg_rmse': [np.mean, np.std]})

            print(csv_name)
            print(df_aggregated.to_string(), '\n')
            #print(df_aggregated.reset_index(drop=True).to_string(), '\n')

if __name__ == '__main__':
    parser = argparse.ArgumentParser(description='Run meta mll hyper-parameter search.')
    parser.add_argument("--dataset", type=str, default='')
    parser.add_argument("--prefix", type=str, default='')
    args = parser.parse_args()
    main(args)