from config import DATA_DIR
import tensorflow as tf
import os
import argparse
import copy

from pacoh import datasets as datasets, algorithms as algorithms
from pacoh.plots_meta_regression import get_best_config_from_csv

HPARAM_EXP_DIR = os.path.join(DATA_DIR, 'tune-hparam-pacoh')
DIR = os.path.dirname(os.path.abspath(__file__))

def main(config):
    """ --- process config ---- """
    posterior_config = config['posterior_config']
    config['posterior_config'] = posterior_config = posterior_config[posterior_config['type']]

    prior_config = copy.deepcopy(posterior_config)
    prior_config['n_batched_priors'] = 1
    config['prior_config'] = prior_config

    if 'activation_fn' in config['model_config']['fc']:
        activation_fn = config['model_config']['fc']['activation_fn']
        del config['model_config']['fc']['activation_fn']
        config['model_config']['fc']['activations'] = [activation_fn for _ in range(4)] + [None]

    data_config = config.pop('data_config')

    n_batched_models_train = posterior_config['n_samples']
    n_batched_models_test = posterior_config['n_samples']

    """ --- setup datasets and samplers ---- """
    if data_config['type'] == 'reg':
        train_ds, val_ds = datasets.get_regression_datasets(data_config,
                                                            n_batched_models_train=n_batched_models_train,
                                                            n_batched_models_test=n_batched_models_test)
    elif data_config['type'] == 'class':
        train_ds, val_ds = datasets.get_classification_datasets(data_config,
                                                                n_batched_models_train=n_batched_models_train,
                                                                n_batched_models_test=n_batched_models_test)
    else:
        raise Exception("Unexpected dataset type")

    n_batched_models_train *= train_ds.n_tasks
    config['posterior_config']['n_batched_priors'] = train_ds.n_tasks
    config['n_batched_models'] = n_batched_models_train

    meta_model = algorithms.MetaBayesianNeuralNetwork_MLAP(meta_train_sampler=train_ds,
                                                           meta_test_sampler=val_ds, **config)
    meta_model.meta_fit()

    train_ds.plot_prediction_functions(meta_model, plot_pred_lines=True, plot_pred_std=True,
                                       sample_functions=True, sample_from_prior=True, plot_data=True)

if __name__ == '__main__':
    parser = argparse.ArgumentParser(description='Run meta mll hyper-parameter search.')
    parser.add_argument("--num_train_tasks", type=int, default='3')
    args = parser.parse_args()

    """ --- read config ---- """
    _csv_name = 'mlap_updated_delta_sin_20_mlap_seeger_mse_svgd_Apr_11_2020_04:56:19.csv'
    _criterion = 'avg_ll'
    csv_path = os.path.join(HPARAM_EXP_DIR, _csv_name)

    config = get_best_config_from_csv(csv_path, _criterion)

    """ --- debugging options  ---- """
    debug = False

    if debug:
        tf.config.experimental_run_functions_eagerly(True)
        config['meta_train_config']['n_total_iterations'] = 20

    """ --- set dataset settings ---- """
    config['data_config']['dataset'] = 'sin'
    config['data_config']['num_train_tasks'] = args.num_train_tasks
    main(config)

