from config import DATA_DIR
import tensorflow as tf
import argparse
import os

from pacoh import datasets as datasets, algorithms as algorithms
from pacoh.plots_meta_regression import get_best_config_from_csv

HPARAM_EXP_DIR = os.path.join(DATA_DIR, 'tune-hparam-pacoh')
DIR = os.path.dirname(os.path.abspath(__file__))

# config = {
#   "data_config": {
#     "batch_size": 5,
#     "dataset": "sin",
#     "meta_batch_size": 5,
#     "num_context_samples": 10,
#     "num_test_valid_samples": None,
#     "num_test_valid_tasks": 2,
#     "num_train_samples": 10,
#     "num_train_tasks": None,
#     "posterior_inference_batch_size": 1,
#     "standardization": True,
#     "tiled": True,
#     "type": "reg"
#   },
#   "hyper_posterior_config": {
#     "particle": {
#       "n_samples": 1
#     },
#     "type": "particle"
#   },
#   "hyper_prior_config": {
#     "bias_log_var_std": 0.4438540478625312,
#     "bias_mean_std": 0.4438540478625312,
#     "kernel_log_var_std": 0.4438540478625312,
#     "kernel_mean_std": 0.4438540478625312,
#     "likelihood_log_var_log_var_mean": -2,
#     "likelihood_log_var_log_var_std": 0.2,
#     "likelihood_log_var_mean_mean": -4,
#     "likelihood_log_var_mean_std": 1,
#     "log_var_mean": -3.1099825124577563,
#     "mean_mean": 0.0
#   },
#   "likelihood_config": {
#     "class": {
#       "learn_likelihood_variables": True
#     },
#     "reg": {
#       "learn_likelihood_variables": False,
#       "log_var": -4.631251503547343
#     },
#     "type": "reg"
#   },
#   "model_config": {
#     "cnn": {},
#     "fc": {
#       "activations": [
#         "relu",
#         "relu",
#         "relu",
#         "relu",
#         None
#       ],
#       "input_dim": None,
#       "layer_units": [
#         32,
#         32,
#         32,
#         32,
#         1
#       ],
#       "output_dim": 1,
#       "activation_fn": "tanh"
#     },
#     "type": "fc"
#   },
#   "posterior_config": {
#     "particle": {
#       "n_samples": 5,
#       "use_init_std": False
#     },
#     "type": "particle"
#   },
#   "prior_config": {
#     "gaussian": {
#       "add_noise": True,
#       "init_from_models": True,
#       "init_std": 0.1,
#       "likelihood": {
#         "log_var_log_var": 0.0,
#         "log_var_mu": -9.210340371976182
#       },
#       "model": {
#         "bias_log_var_mu": -5,
#         "mean_mu": 0.0,
#         "weight_log_var_mu": -5
#       },
#       "n_samples": 10,
#     },
#     "type": "gaussian"
#   },
#   "meta_test_config": {
#     "bandwidth": 83.47088984399889,
#     "learning_rate": 0.0008217670793401941,
#     "n_eval_runs": 0,
#     "n_total_iterations": 3000,
#     "n_train_loss_logs": 5,
#     "optimizer": "adam",
#     "plot_functions": True,
#     "plot_pred_lines": True,
#     "plot_pred_std": True,
#     "prior_prob_weight": 7.911590943566564e-05
#   },
#   "meta_train_config": {
#     "bandwidth": 83.47088984399889,
#     "learning_rate": 0.004661891563825031,
#     "n_eval_runs": 0, #2,
#     "n_total_iterations": 8000,
#     "n_train_loss_logs": 10,
#     "logsumexp_estimate": True,
#     "optimizer": "adam",
#     "plot_functions": False,
#     "plot_pred_lines": False,
#     "plot_pred_std": False,
#     "prior_prob_weight": 0.010651054314222778 / 100
#   },
#
# }

def main(config):
    """ --- process config ---- """
    prior_config = config['prior_config']
    config['prior_config'] = prior_config = prior_config[prior_config['type']]

    posterior_config = config['posterior_config']
    config['posterior_config'] = posterior_config = posterior_config[posterior_config['type']]

    hyper_posterior_config = config['hyper_posterior_config']
    config['hyper_posterior_config'] = hyper_posterior_config = hyper_posterior_config[hyper_posterior_config['type']]

    if 'activation_fn' in config['model_config']['fc']:
        activation_fn = config['model_config']['fc']['activation_fn']
        del config['model_config']['fc']['activation_fn']
        config['model_config']['fc']['activations'] = [activation_fn for _ in range(4)] + [None]

    data_config = config.pop('data_config')

    """ --- setup datasets and samplers ---- """

    n_batched_models_train = hyper_posterior_config['n_samples'] * prior_config['n_samples']
    n_batched_models_test = hyper_posterior_config['n_samples'] * posterior_config['n_samples']

    if data_config['type'] == 'reg':
        train_ds, val_ds = datasets.get_regression_datasets(data_config,
                                                            n_batched_models_train=n_batched_models_train,
                                                            n_batched_models_test=n_batched_models_test)
    elif data_config['type'] == 'class':
        train_ds, val_ds = datasets.get_classification_datasets(data_config,
                                                                n_batched_models_train=n_batched_models_train,
                                                                n_batched_models_test=n_batched_models_test)
    else:
        raise Exception("Unexpected dataset type")

    print("======= Training with: ", train_ds.n_tasks, "tasks")
    meta_model = algorithms.MetaBayesianNeuralNetwork_SVGD(meta_train_sampler=train_ds,
                                                           meta_test_sampler=val_ds, **config)
    meta_model.meta_fit()

    train_ds.plot_prediction_functions(meta_model, plot_pred_lines=True, plot_pred_std=True,
                                       sample_functions=True, sample_from_prior=True, plot_data=True)

if __name__ == '__main__':
    parser = argparse.ArgumentParser(description='Run meta mll hyper-parameter search.')
    parser.add_argument("--num_train_tasks", type=int, default='5')
    args = parser.parse_args()

    """ --- read config ---- """
    _csv_name = 'reg_fix_v0_sin_20_pacoh_map_learned_ll_svgd_Apr_17_2020_03:43:07.csv'
    _criterion = 'avg_ll'
    csv_path = os.path.join(HPARAM_EXP_DIR, _csv_name)

    config = get_best_config_from_csv(csv_path, _criterion)

    """ --- debugging options  ---- """
    debug = False

    if debug:
        tf.config.experimental_run_functions_eagerly(True)
        config['meta_train_config']['n_total_iterations'] = 20

    """ --- set dataset settings ---- """
    config['data_config']['dataset'] = 'sin'
    config['data_config']['num_train_tasks'] = args.num_train_tasks
    main(config)

