import matplotlib.pyplot as plt
import numpy as np
import os

# python launch_mem_profiling --exp_id meta_test_only_log_MLAP --algorithm MLAP
# python launch_mem_profiling --exp_id meta_test_only_log_pacoh_SVGD --algorithm SVGD

ids = ['meta_test_only_log', 'meta_test_only_log_pacoh_SVGD']
DIR = os.path.dirname(os.path.abspath(__file__))

plt.rc('text', usetex=True)
plt.rc('font', family='serif')

dir = '../../../test'

max_only = True

max_mem = -1
max_time = -1

if max_only:
    fig, axs = plt.subplots(1, 2, sharex=True, figsize=(5.5, 2.4))
else:
    fig, ax = plt.subplots(1, 1, figsize=(5, 5))
    axs = [ax]

for id in ids:
    files = os.listdir(dir)
    files = [file for file in files if file.startswith("{}_mem".format(id)) and file.endswith('.dat')]

    mem = []
    time = []

    for file in files:
        n_tasks = int(file.split('_')[-1].split('.')[0])
        if n_tasks > 80:
            continue

        data = np.loadtxt('../../../test/{}'.format(file), skiprows=1, usecols=(1, 2))
        data[:, 1] -= (data[0][1])

        if not max_only:
            # if we are plotting the memory consumption over time, plot it
            axs[0].plot(data[:, 1] / 3600, data[:, 0] / 1000, label='{} tasks'.format(n_tasks))
            # ax.scatter(data[:, 1], data[:, 0], marker='x', s=15, c='k', alpha=0.7)

        # record the maximum amount of memory and the execution time of this run
        mem.append((n_tasks, data[:, 0].max() / 1000))
        time.append((n_tasks, data[:, 1].max() / 3600))

        # save the maximum memory and maximum execution time across runs (for plot window positioning)
        max_mem = max(max_mem, mem[-1][1])
        max_time = max(max_time, time[-1][1])

    if max_only:
        # plot the maximum memory consumption per run
        mem = sorted(mem, key=lambda x: x[0])
        time = sorted(time, key=lambda x: x[0])

        mem = np.array(mem)
        time = np.array(time)

        if 'SVGD' in id:
            axs[0].plot(mem[:, 0], mem[:, 1], linestyle='-', marker='x',
                        label='PACOH-NN')
            axs[1].plot(time[:, 0], time[:, 1], linestyle='-', marker='x', )
        else:
            axs[0].plot(mem[:, 0], mem[:, 1], linestyle='--', marker='.',
                        label='MLAP')
            axs[1].plot(time[:, 0], time[:, 1], linestyle='--', marker='.', )

        #axs[0].scatter(mem[:, 0], mem[:, 1], marker='v', s=15, c='k', alpha=0.7)

        #axs[1].scatter(time[:, 0], time[:, 1], marker='x', s=15, c='k', alpha=0.7)

""""----- set labels, limits and title ------"""

if max_only:
    axs[0].set_xlabel('number of tasks')
    axs[0].set_ylabel('max. memory usage (GB)')
    axs[1].set_xlabel('number of tasks')
    axs[1].set_ylabel('execution time (hours)')

    axs[0].set_ylim((0, max_mem * 1.05))
    axs[1].set_ylim((0, max_time * 1.05))

    axs[0].set_xlim((0, mem[-1][0] * 1.05))

    axs[0].set_title('Memory')
    axs[1].set_title('Execution time')

    # handles, labels = axs[-1].get_legend_handles_labels()
    # axs[-1].legend(handles, labels)
else:
    axs[0].legend()
    axs[0].set_xlim(0, max_time * 1.05)
    axs[0].set_ylim(0, max_mem * 1.05)

    axs[0].set_xlabel('Execution time (in hours)')
    axs[0].set_ylabel('Memory consumption (in GB)')


axs[0].legend()
plt.tight_layout(rect=(0, 0, 1, 0.98))
#plt.show()
fig.savefig(os.path.join(DIR, f'mem_time_requirements_80.pdf'))
fig.savefig(os.path.join(DIR, '/home/jonasrothfuss/Dropbox/Eigene_Dateien/ETH/02_Projects/07_Meta-Learning_NNs/LaTex/figures/mem_time_requirements_80.pdf'))
