import numpy as np

NUM_TRAIN_TASKS = 100
NUM_TEST_VALID_TASKS = 40
NUM_TRAIN_SAMPLES = -1
NUM_CONTEXT_SAMPLES = 25
NUM_TEST_VALID_SAMPLES = 50
DATASET = 'omniglot'
N_CLASSES = 5


likelihood_config = {'class': {'learn_likelihood_variables': False},
                     'type': 'class'
                     }

LAYER_SIZE = 64
N_LAYERS = 4
ACTIVATION = 'relu'

model_config = {'fc': {'input_dim': None,
                       'output_dim': 1 if N_CLASSES == 2 else N_CLASSES,
                       'layer_units': [LAYER_SIZE for _ in range(N_LAYERS)] + [N_CLASSES],
                        'activations': [ACTIVATION for _ in range(N_LAYERS)] + ['softmax']
                       },
                'cnn': {'activations': [ACTIVATION, 'sigmoid' if N_CLASSES == 2 else 'softmax'],
                        'output_dim': 1 if N_CLASSES == 2 else N_CLASSES},
                'type': 'cnn'}

data_config = {'dataset': DATASET,
               'num_train_tasks': NUM_TRAIN_TASKS,
               'num_test_valid_tasks': NUM_TEST_VALID_TASKS,
               'num_train_samples': NUM_TRAIN_SAMPLES,
               'num_context_samples': NUM_CONTEXT_SAMPLES,
               'num_test_valid_samples': NUM_TEST_VALID_SAMPLES,
               'ignore_alphabet_boundaries': True,
               'standardization': False,
               'tiled': True,
               'flatten_data': False,
               'batch_size': 20,
               'posterior_inference_batch_size': -1,
               'meta_batch_size': 5
               }

meta_train_config = {'n_total_iterations': 20000,
                   'n_train_loss_logs': 100,
                   'n_eval_runs': 20,
                   'optimizer': 'adam',
                   'learning_rate': 5e-4,
                   'inner_learning_rate': 0.01,
                   'n_inner_steps': 2,
                   'first_order': True,
                   'split_batch': True
                   }


meta_test_config = {'n_optimization_steps': 1}


if __name__ == "__main__":
    from pacoh.algorithms import MAML
    import pacoh.datasets as datasets
    import tensorflow as tf

    #tf.config.experimental_run_functions_eagerly(True)

    # meta_train_sampler, meta_test_sampler = datasets.get_classification_datasets(data_config,
    #                                                                              n_batched_models_train=False,
    #                                                                              n_batched_models_test=False)


    from pacoh.datasets.classification_datasets import OmniglotMetaDataset
    from pacoh.datasets.data_sampler import MetaDatasetSampler

    dataset = OmniglotMetaDataset(n_classes=N_CLASSES, ignore_alphabet_boundaries=True,
                                  random_state=np.random.RandomState(28))

    # data_train = dataset.generate_meta_train_data(
    #     n_tasks=300,
    #     n_samples=-1)

    data_valid = dataset.generate_meta_test_data(
        n_tasks=30,
        n_samples_context=25,
        n_samples_test=-1
    )

    # # TODO remove
    data_train = []
    for x_context, y_context, x_val, y_val in data_valid:
        x = tf.concat([x_context, x_val], axis=0)
        y = tf.concat([y_context, y_val], axis=0)
        data_train.append((x,y))
    data_valid = data_valid[:30]
    # B) Create Batch Sampler

    meta_train_sampler = MetaDatasetSampler(data_train, per_task_batch_size=25,
                                            meta_batch_size=5, n_batched_models=None,
                                            tiled=False, flatten_data=False, standardization=False)

    meta_test_sampler = MetaDatasetSampler(data_valid, per_task_batch_size=25,
                                           meta_batch_size=-1, n_batched_models=None,
                                           tiled=False, flatten_data=False, standardization=False)
    # copy image attributes (height, width, depth) into samplers
    # img_attrs = dataset.get_img_attributes()
    meta_train_sampler.set_original_shape(dataset.original_shape)
    meta_test_sampler.set_original_shape(dataset.original_shape)

    meta_reg_model = MAML(meta_train_sampler, meta_test_sampler, model_config=model_config, #TODO meta train_sampler
                          likelihood_config=likelihood_config, meta_train_config=meta_train_config,
                          meta_test_config=meta_test_config, random_seed=276)


    meta_reg_model.meta_fit()
