import numpy as np

NUM_TRAIN_TASKS = 100
NUM_TEST_VALID_TASKS = 40
NUM_TRAIN_SAMPLES = -1
NUM_CONTEXT_SAMPLES = 25
NUM_TEST_VALID_SAMPLES = 50
DATASET = 'omniglot'
N_CLASSES = 5


likelihood_config = {'class': {'learn_likelihood_variables': False},
                     'type': 'class'
                     }

LAYER_SIZE = 64
N_LAYERS = 4
ACTIVATION = 'relu'

model_config = {'cnn': {'activations': ['relu', 'softmax'], 'output_dim': 5},
                'fc': {'activations': ['relu', 'relu', 'relu', 'relu', 'sigmoid'],
                       'input_dim': None, 'layer_units': [64, 64, 64, 64, 5],
                       'output_dim': 5},
                'type': 'cnn'}

data_config = {'batch_size': 25,
               'dataset': 'omniglot_5shot_full',
               'meta_batch_size': 5,
               'num_context_samples': None,
               'num_test_valid_samples': None,
               'num_test_valid_tasks': None,
               'num_train_samples': None,
               'num_train_tasks': None,
               'posterior_inference_batch_size': -1,
               'standardization': False,
               'tiled': False,
               'type': 'class'}

meta_train_config = {'first_order': False,
                     'inner_learning_rate': 0.18161393517134333,
                     'learning_rate': 0.00010876781341830915,
                     'n_eval_runs': 10,
                     'n_inner_steps': 5,
                     'n_total_iterations':  100000,
                     'n_train_loss_logs': 20,
                     'optimizer': 'adam',
                     'split_batch': True}

meta_test_config = {'n_optimization_steps': 5}


if __name__ == "__main__":
    from pacoh.algorithms import MAML
    import pacoh.datasets as datasets
    import tensorflow as tf

    #tf.config.experimental_run_functions_eagerly(True)

    # meta_train_sampler, meta_test_sampler = datasets.get_classification_datasets(data_config,
    #                                                                              n_batched_models_train=False,
    #                                                                              n_batched_models_test=False)

    for seed in [33, 34, 35, 36]:
        from pacoh.datasets.data_sampler import get_classification_datasets

        train_ds, val_ds = get_classification_datasets(data_config)

        meta_model = MAML(train_ds, val_ds, model_config=model_config,
                              likelihood_config=likelihood_config, meta_train_config=meta_train_config,
                              meta_test_config=meta_test_config, random_seed=seed)


        meta_model.meta_fit()


        eval_metrics_mean, eval_metrics_std = meta_model.meta_eval(print_results=True)

        print('-----------------------------------')
        print('seed: %i'%seed)
        for key in eval_metrics_mean.keys():
            print('%s: %.4f +- %.4f'%(key, eval_metrics_mean[key], eval_metrics_std[key]))
