import numpy as np


NUM_TRAIN_TASKS = None
NUM_TEST_VALID_TASKS = None
NUM_TRAIN_SAMPLES = None
NUM_CONTEXT_SAMPLES = None
NUM_TEST_VALID_SAMPLES = None
DATASET = 'cauchy_20'



likelihood_config = {'class': {'learn_likelihood_variables': False},
                     'reg': {'log_var': 2 * np.log(0.1),
                             'learn_likelihood_variables': True},
                     'type': 'reg'
                     }

LAYER_SIZE = 64
N_LAYERS = 4
ACTIVATION = 'relu'
OUTPUT_DIM = 1

model_config = {'fc': {'input_dim': None,
                       'output_dim': OUTPUT_DIM,
                       'layer_units': [LAYER_SIZE for _ in range(N_LAYERS)] + [OUTPUT_DIM],
                       'activations': [ACTIVATION for _ in range(N_LAYERS)] + [None]
                       },
                'cnn': {},
                'type': 'fc'}

model_gaussian_prior = {'mean_mu': 0.0,
                        'bias_log_var_mu': 2 * np.log(2.0),
                        'weight_log_var_mu': 2 * np.log(1.0)
                        }

likelihood_gaussian_prior = {'log_var_mu': 2 * np.log(0.1),
                             'log_var_log_var': 2 * np.log(0.5)
                             }

prior_config = { 'model': model_gaussian_prior,
                 'likelihood': likelihood_gaussian_prior,
                 'init_std': 0.1,
                 'n_batched_priors': 1,
                 'add_noise': False,
                }

data_config = {'dataset': DATASET,
               'num_train_tasks': NUM_TRAIN_TASKS,
               'num_test_valid_tasks': NUM_TEST_VALID_TASKS,
               'num_train_samples': NUM_TRAIN_SAMPLES,
               'num_context_samples': NUM_CONTEXT_SAMPLES,
               'num_test_valid_samples': NUM_TEST_VALID_SAMPLES,
               'standardization': True,
               'tiled': True,
               'batch_size': -1,
               'posterior_inference_batch_size': -1,
               'meta_batch_size': 5
               }

meta_train_config = {'n_total_iterations': 20000,
                   'n_train_loss_logs': 40,
                   'n_eval_runs': 10,
                   'n_particles': 5,
                   'bandwidth': 0.01,
                   'optimizer': 'adam',
                   'learning_rate': 0.001,
                   'inner_learning_rate': 0.01,
                   'n_inner_steps': 2, #TODO set back to 2
                   'n_leader_steps_extra': 1,
                   'leader_lr_prefactor': 0.5,
                   'chaser_loss': True,
                   'split_batch': True,
                   'prior_factor': 0.001,
                   }

meta_test_config = {'n_steps_extra': 0}


if __name__ == "__main__":
    from pacoh.algorithms.BMAML import BMAML
    import pacoh.datasets as datasets
    import tensorflow as tf

    #tf.config.experimental_run_functions_eagerly(True)

    # n_tasks = 1050
    # tasks = []
    # for i in range(n_tasks):
    #     x = np.random.uniform(-1, 1, size=(110, 1))
    #     amp = np.random.uniform(0.3, 0.6)
    #     phase = np.random.uniform(0.7, 1.3)
    #     y = amp * np.sin(phase * 2 * np.pi * x) + np.random.normal(scale=0.02, size=x.shape)
    #     tasks.append((x[:40], y[:40], x[40:], y[40:]))
    # data_train = [(x, y) for x, y, _, _, in tasks[:1000]]
    # data_test = tasks[-20:]


    n_particles = meta_train_config['n_particles']

    # meta_train_sampler = datasets.MetaDatasetSampler(data_train, per_task_batch_size=-1, meta_batch_size=2,
    #                                                  standardization=True, tiled=True, n_batched_models=n_particles,
    #                                                  flatten_data=False)
    #
    # meta_test_sampler = datasets.MetaDatasetSampler(data_test, per_task_batch_size=-1, standardization=True,
    #                                                 tiled=True, n_batched_models=n_particles,
    #                                                  flatten_data=False)

    meta_train_sampler, meta_test_sampler = datasets.get_regression_datasets(data_config,
                                                                             n_batched_models_train=n_particles,
                                                                             n_batched_models_test=n_particles)

    meta_reg_model = BMAML(meta_train_sampler, meta_test_sampler, model_config=model_config,
                          likelihood_config=likelihood_config, prior_config=prior_config,
                           meta_train_config=meta_train_config, meta_test_config=meta_test_config,
                          random_seed=276)


    meta_reg_model.meta_fit()