import tensorflow as tf
import numpy as np

import pacoh.datasets as datasets
import pacoh.algorithms as algorithms

import numpy as np
import copy

NUM_TRAIN_TASKS = 4# TODO: 20
NUM_TEST_VALID_TASKS = 6
NUM_TRAIN_SAMPLES = 10
NUM_CONTEXT_SAMPLES = 10
NUM_TEST_VALID_SAMPLES = 100
DATASET = 'physionet_0'

model_config = {'fc': {'input_dim': None,
                       'output_dim': 1,
                       'layer_units': [32, 32, 32, 32, 1],
                       'activations': ['tanh', 'tanh', 'tanh', 'tanh', None]
                       },
                'cnn': {},
                'type': 'fc'}

likelihood_config = {'class': {'learn_likelihood_variables': False},
                     'reg': {'log_var': 2 * np.log(np.sqrt(2)),
                             'learn_likelihood_variables': False,
                             },
                     'type': 'reg',
                     'use_mse_as_loss': True
                     }
# Covered
# std <=> kappa_Pscript
hyper_prior_config = {'mean': 0.0,
                      'std': 4064.6619982881634}

# Covered
# std <=> kappa_Qscript
hyper_posterior_config = {'mean_mu': 0.0,
                          'log_var_mu': -6.0,
                          'std': 0.0001444503653271869,
                          'init_std': 0.01,
                          }

model_gaussian_prior = {'mean_mu': 0.0,
                        'bias_log_var_mu': -6,  # 2 * np.log(1.0), # mean for the log_var of the bias
                        'weight_log_var_mu': -6  # 2 * np.log(0.5)
                        }

likelihood_gaussian_prior = {'log_var_mu': 2 * np.log(0.01),
                             'log_var_log_var': 2 * np.log(2.0)
                             }

data_config = {'dataset': DATASET,
               'num_train_tasks': NUM_TRAIN_TASKS,
               'num_test_valid_tasks': NUM_TEST_VALID_TASKS,
               'num_train_samples': NUM_TRAIN_SAMPLES,
               'num_context_samples': NUM_CONTEXT_SAMPLES,
               'num_test_valid_samples': NUM_TEST_VALID_SAMPLES,
               'standardization': True,
               'tiled': True,
               'batch_size': 5,
               'posterior_inference_batch_size': 5,
               'meta_batch_size': -1  # -1 #[5 to 10]
               }

posterior_config = {'gaussian': {'model': model_gaussian_prior,
                                 'likelihood': likelihood_gaussian_prior,
                                 'init_std': 0.1,
                                 'add_noise': True,
                                 'init_from_models': True,
                                 'n_samples': 5
                                 },
                    'type': 'gaussian'
                    }

meta_train_config = {'epochs': 3000,
                     'n_train_loss_logs': 2,
                     'n_eval_runs': 0,
                     'optimizer': 'adam',
                     'learning_rate': 0.01,  #
                     'plot_functions': False,
                     'plot_pred_lines': True,
                     'plot_pred_std': True,

                     'variance_lr_scale': 5.0,  #
                     'prior_lr_scale': 1.0,  #
                     'task_complexity_weight': 0.26870006891471826,  #
                     'meta_complexity_weight': 0.08033980400235861,  #
                     'bound_type': 'seeger',
                     'delta': 0.005564546625796155,
                     }

meta_test_config = {'epochs': 3000,
                    'n_train_loss_logs': 10,
                    'n_eval_runs': 0,
                    'optimizer': 'adam',
                    'learning_rate': 0.005,  #
                    'plot_functions': False,
                    'plot_pred_lines': True,
                    'plot_pred_std': True,

                    'variance_lr_scale': 2.0,
                    'task_complexity_weight': 0.26870006891471826,  #
                    'bound_type': 'seeger',
                    'delta': 0.005564546625796155,
                    }

posterior_config = posterior_config[posterior_config['type']]

base_config = {'model_config': model_config,
          'likelihood_config': likelihood_config,
          'data_config': data_config,
          'hyper_prior_config': hyper_prior_config,
          'hyper_posterior_config': hyper_posterior_config,
          'posterior_config': posterior_config,
          'meta_train_config': meta_train_config,  # TODO: update
          'meta_test_config': meta_test_config,  # TODO: update
          'random_seed': None
          }


def main():
    # tf.config.experimental_run_functions_eagerly(True)

    # setup dataset and corresponding samplers
    posterior_config = base_config['posterior_config']
    # config['posterior_config'] = posterior_config = posterior_config[posterior_config['type']]

    prior_config = copy.deepcopy(posterior_config)
    prior_config['n_batched_priors'] = 1
    base_config['prior_config'] = prior_config

    n_batched_models_train = posterior_config['n_samples']
    n_batched_models_test = posterior_config['n_samples']

    data_config = base_config['data_config']
    train_ds, val_ds = datasets.get_regression_datasets(data_config,
                                                        n_batched_models_train=n_batched_models_train,
                                                        n_batched_models_test=n_batched_models_test)

    tf.print("data samplers ready")
    del base_config['data_config']

    n_batched_models_train *= train_ds.n_tasks
    base_config['posterior_config']['n_batched_priors'] = train_ds.n_tasks
    base_config['n_batched_models'] = n_batched_models_train

    meta_reg_model = algorithms.MetaBayesianNeuralNetwork_MLAP(train_ds, val_ds, **base_config)

    train_ds.plot_prediction_functions(meta_reg_model, plot_pred_lines=True, sample_functions=True,
                                       sample_from_prior=True, plot_data=True)

    meta_reg_model.meta_fit()

    meta_reg_model.meta_eval(print_results=True)
    train_ds.plot_prediction_functions(meta_reg_model, plot_pred_lines=True, sample_functions=True,
                                       sample_from_prior=True, plot_data=True)
if __name__ == '__main__':
    main()