import tensorflow as tf
import numpy as np
import math
import torch
from datetime import datetime
import argparse
import csv
import sys
import os
parent_path = os.path.dirname(os.path.realpath(__file__))
grandparent_path = os.path.dirname(parent_path)
sys.path.append(grandparent_path)

import pacoh.datasets as datasets
import pacoh.algorithms as algorithms

def write_to_log(message, prm, mode='a', update_file=True):
    # mode='a' is append
    # mode = 'w' is write new file
    if not isinstance(message, list):
        message = [message]
    # update log file:
    if update_file:
        log_file_path = os.path.join(prm.result_dir, 'log') + '.out'
        with open(log_file_path, mode) as f:
            for string in message:
                print(string, file=f)
    # print to console:
    for string in message:
        print(string)

def create_result_dir(prm):
    # If run_name empty, set according to time
    time_str = datetime.now().strftime(' %Y-%m-%d %H:%M:%S')
    if prm.run_name == '':
        prm.run_name = time_str
    prm.result_dir = os.path.join('saved', prm.run_name)
    # prm.result_dir = os.path.join('saved')
    if not os.path.exists(prm.result_dir):
        os.makedirs(prm.result_dir)
    message = ['Log file created at ' + time_str,
               'Run script: ' + sys.argv[0],
               'Parameters:', str(prm), '-'*50]
    write_to_log(message, prm, mode='a') # create new log file

config = {
  "data_config": {
    "batch_size": 5,
    "dataset": "sin_20",
    "meta_batch_size": 4,
    "num_context_samples": 20,
    "num_test_valid_samples": None,
    "num_test_valid_tasks": 2,
    "num_train_samples": 20,
    "num_train_tasks": None,
    "posterior_inference_batch_size": 5,
    "standardization": True,
    "tiled": True,
    "type": "reg"
  },
  "hyper_posterior_config": {
    "particle": {
      "n_samples": 3
    },
    "type": "particle"
  },
  "hyper_prior_config": {
    "bias_log_var_std": 0.4438540478625312,
    "bias_mean_std": 0.4438540478625312,
    "kernel_log_var_std": 0.4438540478625312,
    "kernel_mean_std": 0.4438540478625312,
    "likelihood_log_var_log_var_mean": -2,
    "likelihood_log_var_log_var_std": 0.2,
    "likelihood_log_var_mean_mean": -4,
    "likelihood_log_var_mean_std": 1,
    "log_var_mean": -3.1099825124577563,
    "mean_mean": 0.0
  },
  "likelihood_config": {
    "class": {
      "learn_likelihood_variables": True
    },
    "reg": {
      "learn_likelihood_variables": False,
      "log_var": -4.631251503547343
    },
    "type": "reg"
  },
  "model_config": {
    "cnn": {},
    "fc": {
      "activations": [
        "relu",
        "relu",
        "relu",
        "relu",
        None
      ],
      "input_dim": None,
      "layer_units": [
        32,
        32,
        32,
        32,
        1
      ],
      "output_dim": 1,
      "activation_fn": "tanh"
    },
    "type": "fc"
  },
  "posterior_config": {
    "particle": {
      "n_samples": 5,
      "use_init_std": False
    },
    "type": "particle"
  },
  "prior_config": {
    "gaussian": {
      "add_noise": True,
      "init_from_models": True,
      "init_std": 0.1,
      "likelihood": {
        "log_var_log_var": 0.0,
        "log_var_mu": -9.210340371976182
      },
      "model": {
        "bias_log_var_mu": -5,
        "mean_mu": 0.0,
        "weight_log_var_mu": -5
      },
      "n_samples": 10
    },
    "type": "gaussian"
  },
  "meta_test_config": {
    "bandwidth": 83.47088984399889,
    "learning_rate": 0.0008217670793401941,
    "n_eval_runs": 0,
    "n_total_iterations": 3000,
    "n_train_loss_logs": 100,
    "optimizer": "adam",
    "plot_functions": True,
    "plot_pred_lines": True,
    "plot_pred_std": True,
    "prior_prob_weight": 7.911590943566564e-05
  },
  "meta_train_config": {
    "bandwidth": 83.47088984399889,
    "learning_rate": 0.004661891563825031,
    "n_eval_runs": 2,
    "n_total_iterations": 20000,
    "n_train_loss_logs": 20,
    "logsumexp_estimate": True,
    "optimizer": "adam",
    "plot_functions": True,
    "plot_pred_lines": True,
    "plot_pred_std": True,
    "prior_prob_weight": 0.010651054314222778 / 100
  }  
}
# print(type(config))
### Update Configuration Dictionary
csv_parent_path = os.path.join(grandparent_path,'pacoh_nn_hyperparams')
file_name = os.path.join(csv_parent_path,'final_berkeley_pacoh_svgd_learned_ll_logsumexp_svgd_Aug_04_2020_15:02:33.csv')

with open(file_name,'r',encoding="utf-8") as f:
    reader = csv.reader(f)
    fieldnames = next(reader)
    # print(fieldnames)
    csv_reader = csv.DictReader(f,fieldnames=fieldnames) 
    # print(type(csv_reader)) # csv.DictReader
   

    for row in csv_reader:
        d={}
        for k,v in row.items():
            d[k]=v
        # print(d) 
        break   
    config.update(d)

for k,v in config.items():
  if type(v)== str:
    config[k] = eval(v)
### Update Configuration Dictionary

prior_config = config['prior_config']
config['prior_config'] = prior_config = prior_config[prior_config['type']]

config['posterior_config'].update(config['posterior_config'].pop(config['posterior_config']['type']))
posterior_config = config['posterior_config']

hyper_posterior_config = config['hyper_posterior_config']
config['hyper_posterior_config'] = hyper_posterior_config = hyper_posterior_config[
    hyper_posterior_config['type']]

if 'activation_fn' in config['model_config']['fc']:
    activation_fn = config['model_config']['fc']['activation_fn']
    del config['model_config']['fc']['activation_fn']
    config['model_config']['fc']['activations'] = [activation_fn for _ in range(4)] + [None]

data_config = config.pop('data_config')
model_config = config['model_config']
likelihood_config = config['likelihood_config']
hyper_prior_config = config['hyper_prior_config']
training_config= config['meta_train_config']
inference_training_config = config['meta_test_config']
### hyperparameter


def main(prm):    
    #tf.config.experimental_run_functions_eagerly(True)

    # setup data set and corresponding samplers
    n_batched_models_train = hyper_posterior_config['n_samples'] * prior_config['n_samples']
    n_batched_models_test = hyper_posterior_config['n_samples'] * posterior_config['n_samples']
    train_ds, val_ds = datasets.get_regression_datasets(data_config,  n_batched_models_train=n_batched_models_train,
                                                                  n_batched_models_test=n_batched_models_test,
                                                                  n_samples_to_keep_val=1)

    tf.print("data samplers ready")
    meta_reg_model = algorithms.MetaBayesianNeuralNetwork_SVGD(train_ds, val_ds, model_config, likelihood_config,
                                                               prior_config, posterior_config,
                                                               hyper_prior_config, hyper_posterior_config,
                                                               training_config, inference_training_config,
                                                               plot_prior_during_training=False, random_seed=22, catoni_constant = prm.catoni_constant)

    # train_ds.plot_prediction_functions(meta_reg_model, plot_pred_lines=True, plot_pred_std=True,
                                       # sample_functions=True, sample_from_prior=True, plot_data=False) # original version: plot_data = True
    tf.print('model created')

    meta_reg_model.meta_fit()

    # train_ds.plot_prediction_functions(meta_reg_model, plot_pred_lines=True, sample_functions=True,
                                       # sample_from_prior=True, plot_data=False) # original_version: plot_data = True

    meta_reg_model.meta_eval(print_results=True)

    eval_metrics_mean, eval_metrics_std = meta_reg_model.meta_eval(print_results=True)
    for key in eval_metrics_mean:
        write_to_log("%s: %.4f +- %.4f" % (key, eval_metrics_mean[key], eval_metrics_std[key]), prm)

if __name__ == '__main__':
    for flag in range(1,2):
      parser = argparse.ArgumentParser()

      parser.add_argument('--dataset', type=str, help='environment dataset',
                      default='berkeley')
      parser.add_argument('--method', type=str, help='meta-learning method',
                      default='GOHP-BNN')
      parser.add_argument('--run_name', type=str, help='meta-learning method',
                      default='')
      parser.add_argument('--gpu_index', type=int, help='the ID of GPU',
                      default=1)
      parser.add_argument('--catoni_constant', type = float, help = 'the predefined constant in Catonis bound', default = 11.4)      

      prm = parser.parse_args()
      prm.run_name = prm.dataset + '_' + prm.method + 'constant' + '_' + str(prm.catoni_constant)
      # print(prm.run_name)
      prm.device = torch.device("cuda:"+str(prm.gpu_index) if torch.cuda.is_available() else "cpu")
      create_result_dir(prm)        
      main(prm)  