import numpy as np


NUM_TRAIN_TASKS = None
NUM_TEST_VALID_TASKS = None
NUM_TRAIN_SAMPLES = None
NUM_CONTEXT_SAMPLES = None
NUM_TEST_VALID_SAMPLES = None
DATASET = 'sin_20'



likelihood_config = {'class': {'learn_likelihood_variables': False},
                     'reg': {'log_var': 2 * np.log(np.sqrt(0.5)),
                             'learn_likelihood_variables': False},
                     'type': 'reg'
                     }

LAYER_SIZE = 64
N_LAYERS = 4
ACTIVATION = 'relu'
OUTPUT_DIM = 1

model_config = {'fc': {'input_dim': None,
                       'output_dim': OUTPUT_DIM,
                       'layer_units': [LAYER_SIZE for _ in range(N_LAYERS)] + [OUTPUT_DIM],
                       'activations': [ACTIVATION for _ in range(N_LAYERS)] + [None]
                       },
                'cnn': {},
                'type': 'fc'}

data_config = {'dataset': DATASET,
               'num_train_tasks': NUM_TRAIN_TASKS,
               'num_test_valid_tasks': NUM_TEST_VALID_TASKS,
               'num_train_samples': NUM_TRAIN_SAMPLES,
               'num_context_samples': NUM_CONTEXT_SAMPLES,
               'num_test_valid_samples': NUM_TEST_VALID_SAMPLES,
               'standardization': True,
               'tiled': False,
               'batch_size': -1,
               'posterior_inference_batch_size': -1,
               'meta_batch_size': 10
               }

meta_train_config = {'n_total_iterations': 20000,
                   'n_train_loss_logs': 100,
                   'n_eval_runs': 100,
                   'optimizer': 'adam',
                   'learning_rate': 0.001,
                   'inner_learning_rate': 0.01,
                   'n_inner_steps': 2,
                   'first_order': False,
                    'split_batch': True
                   }


meta_test_config = {'n_optimization_steps': 2}


if __name__ == "__main__":
    from pacoh.algorithms import MAML
    import pacoh.datasets as datasets
    import tensorflow as tf

    #tf.config.experimental_run_functions_eagerly(True)

    n_tasks = 1050
    tasks = []
    for i in range(n_tasks):
        x = np.random.uniform(-1, 1, size=(110,1))
        amp = np.random.uniform(0.3, 0.6)
        phase = np.random.uniform(0.7, 1.3)
        y =  amp * np.sin(phase * 2 * np.pi * x)
        tasks.append((x[:40], y[:40], x[40:], y[40:]))
    data_train = [(x, y) for x, y, _, _, in tasks[:1000]]
    data_test = tasks[-20:]


    meta_train_sampler = datasets.MetaDatasetSampler(data_train, per_task_batch_size=-1, meta_batch_size=5, tiled=False,
                                            standardization=True, flatten_data=False)

    meta_test_sampler = datasets.MetaDatasetSampler(data_test, per_task_batch_size=-1, tiled=False,
                                           flatten_data=False, standardization=True)

    #meta_train_sampler, meta_test_sampler = datasets.get_regression_datasets(data_config)

    meta_reg_model = MAML(meta_train_sampler, meta_test_sampler, model_config=model_config,
                          likelihood_config=likelihood_config, meta_train_config=meta_train_config,
                          meta_test_config=meta_test_config, random_seed=276)


    meta_reg_model.meta_fit()