import pacoh.experiments.hyperparam_search.tuners as tuners
import pacoh.datasets as datasets
import pacoh.algorithms as algorithms
import matplotlib.pyplot as plt
import tensorflow as tf
import os

DIR = os.path.dirname(os.path.abspath(__file__))

def get_base_config(args, max_n_samples, plot, n_samples):
    modelTuner = tuners.get_tuner_class(args)
    config = modelTuner.get_base_config()

    config['data_config']['num_context_samples'] = config['data_config']['num_train_samples'] = max_n_samples
    config['data_config']['num_test_valid_tasks'] = 2
    config['data_config']['batch_size'] = min(config['data_config']['batch_size'], n_samples)
    config['data_config']['posterior_inference_batch_size'] = min(config['data_config']['posterior_inference_batch_size'], n_samples)


    prior_config = config['prior_config']
    config['prior_config'] = prior_config = prior_config[prior_config['type']]

    posterior_config = config['posterior_config']
    config['posterior_config'] = posterior_config = posterior_config[posterior_config['type']]

    config['train_config']['plot_functions'] = plot
    config['train_config']['plot_pred_lines'] = plot
    config['train_config']['plot_pred_std'] = plot
    config['train_config']['n_eval_runs'] = 0
    # config['train_config']['n_total_iterations'] = 20

    return config

def main(args):
    n_samples = [int(i) for i in args['n_samples'].split(',')]
    max_n_samples = max(n_samples)

    fig, axs = plt.subplots(1, len(n_samples), sharey=True, sharex=True, figsize=(10, 4))
    axs = axs.flatten()
    # reg_models = []
    # tasks = []

    for ax, n_s in zip(axs, n_samples):
        config = get_base_config(args, max_n_samples, False, n_s)

        data_config = config.pop('data_config')
        n_batched_models = config['posterior_config']['n_samples']

        _, val_ds = datasets.get_regression_datasets(data_config,
                                                     n_batched_models_train=n_batched_models,
                                                     n_batched_models_test=n_batched_models,
                                                     add_meta_data_to_context=args['add_meta_data_to_context'],
                                                     n_samples_to_keep_val=n_s, seed=123)

        task = val_ds.meta_train_tasks[1]

        # task.plot_data(True, True, ax)

        reg_model = algorithms.BNN_SVGD.BayesianNeuralNetworkSVGD(task, **config)

        reg_model.fit()

        # reg_models.append(reg_model)
        # tasks.append(task)

        task.plot_prediction_functions(model=reg_model, ax=ax,
                                       plot_pred_std=args['plot_pred_std'],
                                       plot_pred_lines=args['plot_pred_std'],
                                       plot_data=True)
        ax.set_xlabel('x')
        ax.set_ylabel('y')
        ax.set_title(f'Training with {n_s} samples')

    axs[0].set_ylim((0, 10))

    handles, labels = axs[-1].get_legend_handles_labels()
    axs[-1].legend(handles, ['Test', 'Train'])

    #
    plt.tight_layout(rect=(0, 0, 1, 0.98))
    plt.show()

    fig.savefig(os.path.join(DIR, 'bnn_sine.png'))
    fig.savefig(os.path.join(DIR, 'bnn_sine.pdf'))


if __name__ == '__main__':
    args = {}
    args['meta_algo_type'] = 'none'
    args['inference_algo_type'] = 'svgd_learned_context'
    args['dataset'] = 'sin'
    args['n_samples'] = '1, 5, 10'
    args['plot_pred_std'] = True
    args['plot_pred_lines'] = True
    # tf.config.experimental_run_functions_eagerly(True)
    main(args)