import tensorflow as tf

import numpy as np

import pacoh.algorithms.BNN_SVGD
import pacoh.datasets.data_sampler


NUM_TRAIN_TASKS = None
NUM_TEST_VALID_TASKS = None
NUM_TRAIN_SAMPLES = None
NUM_CONTEXT_SAMPLES = None
NUM_TEST_VALID_SAMPLES = None
DATASET = 'sin_20'

if DATASET == 'pendulum':
    OUTPUT_DIM = 3
else:
    OUTPUT_DIM = 1

def main():
    likelihood_config = {'class': {'learn_likelihood_variables': False},
                         'reg': {'log_var': 2 * np.log(0.01),
                                 'learn_likelihood_variables': True},
                         'type': 'reg'
                         }

    model_config = {'fc': {'input_dim': None,
                           'output_dim': OUTPUT_DIM,
                           'layer_units': [32, 32, 32, 32, OUTPUT_DIM],
                           'activations': ['relu', 'relu', 'relu', 'relu', None]
                           # 'activations': ['tanh', 'tanh', 'tanh', None]
                           },
                    'cnn': {},
                    'type': 'fc'}

    if 'physionet' in DATASET:
        batch_size = 6
    elif 'sin' in DATASET:
        batch_size = 5
    elif 'cauchy' in DATASET:
        batch_size = 5
    elif 'swissfel' in DATASET:
        batch_size = 20
    elif 'mhc' in DATASET:
        batch_size = 10
    elif 'pendulum' in DATASET:
        batch_size = 50
    elif 'berkeley' in DATASET:
        batch_size = 20
    else:
        raise AssertionError("Unknown dataset")

    data_config = {'dataset': DATASET,
                   'num_train_tasks': NUM_TRAIN_TASKS,
                   'num_test_valid_tasks': NUM_TEST_VALID_TASKS,
                   'num_train_samples': NUM_TRAIN_SAMPLES,
                   'num_context_samples': NUM_CONTEXT_SAMPLES,
                   'num_test_valid_samples': NUM_TEST_VALID_SAMPLES,
                   'standardization': True,
                   'tiled': True,
                   'meta_batch_size': -1,
                   'batch_size': batch_size,
                   'posterior_inference_batch_size': batch_size,
                   }

    model_gaussian_prior = {'mean_mu': 0.0,
                            'bias_log_var_mu': 2 * np.log(1.0),
                            'weight_log_var_mu': 2 * np.log(0.5)
                            }

    likelihood_gaussian_prior = {'log_var_mu': 2 * np.log(0.05),
                                 'log_var_log_var': 2 * np.log(2.0)
                                 }

    prior_config = {'gaussian': {'model': model_gaussian_prior,
                                 'likelihood': likelihood_gaussian_prior,
                                 'init_std': 0.1,
                                 'n_batched_priors': 1,
                                 'add_noise': False
                                 },
                    'type': 'gaussian'
                    }

    posterior_config = {'n_samples': 10,
                        'use_init_std': False,
                        'type': 'particle'
                        }

    training_config = {'n_total_iterations': 10000,
                       'n_train_loss_logs': 20,
                       'n_eval_runs': 10,
                       'optimizer': 'adam',
                       'learning_rate': 0.002,
                       'plot_functions': True,
                       'plot_pred_lines': True,
                       'plot_pred_std': True,
                       'prior_prob_weight': 0.5,
                       'bandwidth': 100,
                       'n_iter_fixed_likelihood': 0,
                       }

    prior_config = prior_config[prior_config['type']]

    #tf.config.experimental_run_functions_eagerly(True)

    # create datasets and corresponding samplers
    n_batched_models = posterior_config['n_samples']
    _, meta_test_sampler = pacoh.datasets.get_regression_datasets(data_config, n_batched_models_test=n_batched_models,
                                                                  n_batched_models_train=n_batched_models)
    dataset_sampler = meta_test_sampler.tasks[0]
    #dataset_sampler.plot_data(False, False)

    reg_model = pacoh.algorithms.BNN_SVGD.BayesianNeuralNetworkSVGD(dataset_sampler, model_config, likelihood_config,
                                                                    prior_config, posterior_config,
                                                                    training_config)
    reg_model.fit()


if __name__ == '__main__':
    main()
