import torch
import os
import sys
import math

BASE_DIR = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
DATA_DIR = os.path.join(BASE_DIR, 'data')
HPARAM_EXP_DIR = os.path.join(DATA_DIR, 'tune-hparam-pacoh')

sys.path.append(BASE_DIR)
N_THREADS_PER_RUN = 2
SEED = 28

for ds in ['sin_20', 'cauchy_20', 'swissfel', 'physionet_0', 'physionet_2', 'berkeley']:
    # 1) load / generate data
    from pacoh.datasets.regression_datasets import provide_data
    data_train, data_valid, _ = provide_data(dataset=ds, seed=SEED)

    # 2) Fit model
    from pacoh_gp.GPR_meta_mll import GPRegressionMetaLearned
    torch.set_num_threads(N_THREADS_PER_RUN)
    model = GPRegressionMetaLearned(data_train, learning_mode='both', covar_module='SE', mean_module='constant',
                                    num_iter_fit=10000)
    model.meta_fit(log_period=500)

    # 3) evaluate on test set
    ll, rmse, calib_err = model.eval_datasets(data_valid)

    print('Dataset: %s'%ds)
    print('ll:', round(ll, 3))
    print('rmse:', round(rmse, 3))
    print('calib_err:', round(calib_err, 3))
