import os
import sys
from datetime import datetime

BASE_DIR = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
sys.path.append(BASE_DIR)

from pacoh_gp.util import generate_launch_commands

import pacoh_gp.meta_map_hyperparam as meta_map_hyperparam
import pacoh_gp.meta_vi_hyperparam as meta_vi_hyperparam
import pacoh_gp.meta_svgd_hyperparam as meta_svgd_hyperparam

MODE = 'oci'
DRY = False

EXP_PREFIX = 'feb23'
NUM_CPUS = 32

command_list = []

for hparam_search_module in [meta_map_hyperparam]:

    exp_config = {
        'dataset': ['swissfel', 'berkeley'],
        'num_cpus': [NUM_CPUS],
        'experiment_prefix': [EXP_PREFIX]
    }
    command_list += generate_launch_commands(hparam_search_module, exp_config, check_flags=False)

print(command_list)

if MODE == 'oci':
    answer = input("About to submit %i compute jobs to the oci cloud. Proceed? [yes/no]\n" % len(command_list))
    if answer == 'yes':
        import oci_launcher.mode as mode
        import oci_launcher.mount as mount
        from oci_launcher.launch import launch_python
        from pacoh.experiments.utils import AsyncExecutor

        def launch_command_oci(command):
            target_file = command.split(" ")[1]
            cmd_args = " ".join(command.split(" ")[2:])

            REMOTE_PYTHON_INTERPRETER = '/home/ubuntu/miniconda3/envs/meta-nn/bin/python -u'

            mode_oci = mode.OCIMode(oci_shape='BM.Standard.E2.64')

            MODE = mode_oci

            mounts = [
                mount.MountLocal(local_dir=BASE_DIR + '/', mount_point='/home/ubuntu/meta-learning-NNs',
                                 pythonpath=True),
                mount.MountDropbox(mount_point='meta-learning-NNs/data', dropbox_path='/meta-learning-NNs_data',
                                   output=True, skip_folders='omniglot_resized,physionet2012,sensor_data,swissfel,2018_10_31')
            ]

            datetime_str = datetime.now().strftime("%d-%m-%y_%H:%M:%S.txt")
            launch_python(target=target_file, mount_points=mounts,
                          target_mount_dir='/home/ubuntu/meta-learning-NNs',
                          mode=MODE, args=cmd_args, verbose=True,
                          python_cmd=REMOTE_PYTHON_INTERPRETER, tmux=True,
                          stdout_file='/home/ubuntu/meta-learning-NNs/data/%s_%s_%s.out'%(EXP_PREFIX, datetime_str, str(command.__hash__())))

        exec = AsyncExecutor(n_jobs=len(command_list))
        exec.run(launch_command_oci, command_list)



elif MODE == 'leonhard':
    cluster_cmds = []
    for python_cmd in command_list:
        bsub_cmd = 'bsub' \
                   ' -W 23:00'\
                   ' -R "rusage[mem=4500]"' \
                   ' -R "rusage[ngpus_excl_p=1]"' \
                   ' -n %i '% (NUM_CPUS)
        cluster_cmds.append(bsub_cmd + ' ' + python_cmd)

    answer = input("About to submit %i compute jobs to the cluster. Proceed? [yes/no]\n"%len(cluster_cmds))
    if answer == 'yes':
        for cmd in cluster_cmds:
            if DRY:
                print(cmd)
            else:
                os.system(cmd)

elif MODE == 'local':
    answer = input("About to run %i compute jobs in a for loop. Proceed? [yes/no]\n"%len(command_list))
    if answer == 'yes':
        for cmd in command_list:
            if DRY:
                print(cmd)
            else:
                os.system(cmd)

else:
    raise NotImplementedError