import os
from datetime import datetime
import sys
import argparse
import math

import torch
import tensorflow as tf

os.environ['CUDA_VISIBLE_DEVICES'] = '1'
def write_to_log(message, prm, mode='a', update_file=True):
    # mode='a' is append
    # mode = 'w' is write new file
    if not isinstance(message, list):
        message = [message]
    # update log file:
    if update_file:
        log_file_path = os.path.join(prm.result_dir, 'log') + '.out'
        with open(log_file_path, mode) as f:
            for string in message:
                print(string, file=f)
    # print to console:
    for string in message:
        print(string)

def create_result_dir(prm):
    # If run_name empty, set according to time
    time_str = datetime.now().strftime(' %Y-%m-%d %H:%M:%S')
    if prm.run_name == '':
        prm.run_name = time_str
    prm.result_dir = os.path.join('saved', prm.run_name)
    # prm.result_dir = os.path.join('saved')
    if not os.path.exists(prm.result_dir):
        os.makedirs(prm.result_dir)
    message = ['Log file created at ' + time_str,
               'Run script: ' + sys.argv[0],
               'Parameters:', str(prm), '-'*50]
    write_to_log(message, prm, mode='a') # create new log file


def main(prm):
    tf.get_logger().setLevel('ERROR')


    from pacoh_nn.datasets.regression_datasets import provide_data

    meta_train_data, meta_test_data, _ = provide_data(dataset=prm.dataset)

    from pacoh_nn.pacoh_nn_regression import PACOH_NN_Regression
    
    # prior_weight_flag = prm.flag * 0.1
    pacoh_model = PACOH_NN_Regression(meta_train_data, prior_weight=0.8, bandwidth=480,
                                      hyper_prior_likelihood_log_var_mean_mean=-2.0,
                                      hyper_prior_log_var_mean=-2, hyper_prior_nn_std=0.1,
                                      hyper_prior_weight=1e-5, lr=1.5e-3, learn_likelihood=True,
                                      random_seed=28, num_iter_meta_train=20000, catoni_constant = prm.catoni_constant)

    pacoh_model.meta_fit(meta_test_data[:10], eval_period=10000, log_period=1000, plot_period=5000)

    eval_metrics_mean, eval_metrics_std = pacoh_model.meta_eval_datasets(meta_test_data)
    for key in eval_metrics_mean:
        write_to_log("%s: %.4f +- %.4f" % (key, eval_metrics_mean[key], eval_metrics_std[key]), prm)


if __name__ == '__main__':
    for flag in range(1,2):
        parser = argparse.ArgumentParser()

        parser.add_argument('--dataset', type=str, help='environment dataset',
                        default='swissfel')
        parser.add_argument('--method', type=str, help='meta-learning method',
                        default='GOHP-BNN')
        parser.add_argument('--run_name', type=str, help='meta-learning method',
                        default='')
        parser.add_argument('--gpu_index', type=int, help='the ID of GPU',
                        default=1)
        parser.add_argument('--catoni_constant', type = float, help = 'the predefined constant in Catonis bound', default = math.e)
        
        prm = parser.parse_args()
        prm.run_name = prm.dataset + '_' + prm.method + 'constant' + '_' + str(prm.catoni_constant)
        # print(prm.run_name)
        prm.flag = flag
        prm.device = torch.device("cuda:"+str(prm.gpu_index) if torch.cuda.is_available() else "cpu")
        
        create_result_dir(prm)        
        main(prm)